#uses "fwViewer/fwViewer_loginClass.ctl"
#uses "fwViewer/fwViewer_edmsFileManager.ctl"

class fwViewer_documentOpener {
  shared_ptr<fwViewer_loginClass> loginManager;
  shared_ptr<fwViewer_edmsFileManager> fileManager;
  const string PdfViewerPanelPath = "fwViewer/fwViewer_pdfPanel.pnl";

  public fwViewer_documentOpener(shared_ptr<fwViewer_loginClass> obj = new fwViewer_loginClass) {
    loginManager = obj;
    fileManager = fwViewer_edmsFileManager::create(loginManager);
  }

  public bool isPdfFile(string url) {
    bool pdfFile = true;
	string fileName = getFileName(url);
    int res = strpos(fileName, ".pdf");
    if(res < 0) {
      pdfFile = false;
    }
    return pdfFile;
  }

   public string getFileName(string url) {
    string clerPath = getClearPath(url);
    return fileManager.getFileName(clerPath);
  }

  public string getPdfFilePath(string url) {
    string path;
    int page;
    string clearPath = getClearPath(url);
    path = getPdfFile(clearPath);

    return path;
  }
  
  public int getPageNumber(string url) {
      int pageNumber = 1;
      string parametersList;
      string searchStr = ".pdf?page=";
      int lengthSearchStr = strlen(searchStr);
	  
      int pos = strpos (url,searchStr);
      if (pos < 0) {
        return  pageNumber;
      }
	  
      pos = pos+lengthSearchStr;
      parametersList = substr(url,pos);
      pos = strpos (parametersList,"&");
      if (pos < 0) {
		 pageNumber = (int)parametersList;
         return  pageNumber;
      }
      pageNumber = (int)substr(parametersList,0,pos);

      return pageNumber;
  }
  
  private string getPdfFile (string url) {
    string path;
    string message ;
    mapping returnValue;
    bool status = true;
    returnValue = fileManager.downloadFile(url);
    status = (bool)returnValue["status"];
    if (!status) {
      message = "Problem with the file: " + url;
      DebugTN(message,returnValue);
      message  ="Problem with the pdfViewer, for more information check The LogViewer. "+
                       "Do you want open the pdf file in a external browser?";
      status = fwViewer_showYesNoWindow(message);
      if (status) {
         dyn_string exceptionInfo;
          fwGeneral_openInExternalBrowser(url, exceptionInfo);
          if (dynlen(exceptionInfo)) {
            fwExceptionHandling_display(exceptionInfo);
          }
      }

    }
    path = returnValue["filePath"];
    return path;
  }
  
	public bool  openPdfViewer(string url) {
		bool status = false;
		string path;
		string clearPath = getClearPath(url);
		path = getPdfFile(clearPath);
		if (path != "") {
			int page = getPageNumber(url);
			status = showPdfFile(path,page,url);
		}
		return status;
	}

  private bool showPdfFile(string path, int page = 1, string url = "") {
    bool status = false;
    mapping returnValue;
    string panelName = "Viewer_";
	string hash = cryptoHash((string)getCurrentTime());
	panelName += hash;
    if (path !=""){
      dyn_string parameters = makeDynString("$path:" + path, "$url:" + url, "$page:" + page);
      ChildPanelOnCentral(PdfViewerPanelPath, panelName, parameters);
    }
    return status;
  }
  
  private string getClearPath(string url) {
    string searchedWord = ".pdf?";
    int pos = strpos(url,searchedWord);
    if (pos < 0) {
      return url;
    }
    int lenSearchWordWithoutQuestionMark = strlen(searchedWord) -1;
    int len =lenSearchWordWithoutQuestionMark + pos;
    string clearUrl = substr(url,0,len);
    return clearUrl;

  }
 
};
