#uses "fwUnitTestComponentAsserts.ctl"

#uses "fwRPC/fwRPC_common.ctl"
#uses "fwRPC/fwRPC_UUID.ctl"

void test_fwRPC_common_makeDpName_returns_valid_name() {
  string name;

  name = fwRPC_common_makeDpName("DPTYPE", "IDSTR");

  assertEqual("DPTYPE/IDSTR", name);
}


void test_fwRPC_common_isValidHdl_checks_for_hdl_validity() {
  assertEqual(false, fwRPC_common_isValidHdl(""));
  assertEqual(false, fwRPC_common_isValidHdl("somestirng"));
  assertEqual(false, fwRPC_common_isValidHdl(349));
  assertEqual(false, fwRPC_common_isValidHdl(makeDynString()));
  assertEqual(true, fwRPC_common_isValidHdl(fwRPC_UUID_generateUUID4()));
}


void test_fwRPC_common_unpackMessage_unpacks_valid_message_strings() {
  string id, type, proj, name, buffer;

  assertEqual(-1, fwRPC_common_unpackMessage("", id, type, proj, name, buffer));
  assertEqual(-1, fwRPC_common_unpackMessage(
    "",
    id, type, proj, name, buffer)
  );
  assertEqual(-1, fwRPC_common_unpackMessage(
    "id",
    id, type, proj, name, buffer)
  );
  assertEqual(-1, fwRPC_common_unpackMessage(
    "id type",
    id, type, proj, name, buffer)
  );
  assertEqual(-1, fwRPC_common_unpackMessage(
    "id type proj",
    id, type, proj, name, buffer)
  );
  assertEqual(-1, fwRPC_common_unpackMessage(
    "id type proj name",
    id, type, proj, name, buffer)
  );

  assertEqual(0, fwRPC_common_unpackMessage(
    "id type proj name buffer",
    id, type, proj, name, buffer)
  );

  assertEqual("id", id);
  assertEqual("type", type);
  assertEqual("proj", proj);
  assertEqual("name", name);
  assertEqual("buffer", buffer);

  assertEqual(0, fwRPC_common_unpackMessage(
    "id type proj name longer buffer",
    id, type, proj, name, buffer)
  );

  assertEqual("longer buffer", buffer);
}


void test_fwRPC_common_cleanProjectName_strips_trailing_colon() {
  string n = "projectname";

  assertEqual(n, fwRPC_common_cleanProjectName(n));
  assertEqual(n, fwRPC_common_cleanProjectName(n+":"));
}


void test_fwRPC_common_isClient_checks_if_name_is_client() {
  assertTrue(fwRPC_dist_isClient(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CLIENT));
  assertTrue(fwRPC_dist_isClient(fwRPC_DIST_TAG_REMOTE + "_" + fwRPC_DIST_TAG_CLIENT));

  assertFalse(fwRPC_dist_isClient(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER));
  assertFalse(fwRPC_dist_isClient(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE));
}


void test_fwRPC_common_isWorker_checks_if_name_is_worker() {
  assertTrue(fwRPC_dist_isWorker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER));
  assertTrue(fwRPC_dist_isWorker(fwRPC_DIST_TAG_REMOTE + "_" + fwRPC_DIST_TAG_WORKER));

  assertFalse(fwRPC_dist_isWorker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CLIENT));
  assertFalse(fwRPC_dist_isWorker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE));
}


void test_fwRPC_common_isCore_checks_if_name_is_core() {
  assertTrue(fwRPC_dist_isCore(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE));
  assertTrue(fwRPC_dist_isCore(fwRPC_DIST_TAG_REMOTE + "_" + fwRPC_DIST_TAG_CORE));

  assertFalse(fwRPC_dist_isCore(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER));
  assertFalse(fwRPC_dist_isCore(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CLIENT));
}


void test_fwRPC_common_isBroker_checks_if_name_is_broker() {
  assertTrue(fwRPC_dist_isBroker(fwRPC_DIST_BRKR));

  assertFalse(fwRPC_dist_isBroker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE));
  assertFalse(fwRPC_dist_isBroker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER));
  assertFalse(fwRPC_dist_isBroker(fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CLIENT));
}


void test_fwRPC_common_nameMatches_checks_if_name_matches_tag() {
  assertTrue(fwRPC_dist_nameMatches(
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE,
    fwRPC_DIST_TAG_CORE
  ));

  assertTrue(fwRPC_dist_nameMatches(
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER,
    fwRPC_DIST_TAG_WORKER
  ));

  assertTrue(fwRPC_dist_nameMatches(
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CLIENT,
    fwRPC_DIST_TAG_CLIENT
  ));
}


void test_fwRPC_common_getIdFromName_retrieves_name_from_identifier() {
  string name = "xyz";

  assertEqual(name, fwRPC_dist_getIdFromName(
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_CORE + "_" + name
  ));

  assertEqual(name, fwRPC_dist_getIdFromName(
    fwRPC_DIST_TAG_REMOTE + "_" + fwRPC_DIST_TAG_CORE + "_" + name
  ));

  assertEqual(name, fwRPC_dist_getIdFromName(
    fwRPC_DIST_TAG_LOCAL + "_" + fwRPC_DIST_TAG_WORKER + "_" + name
  ));
}
