#uses "fwUnitTestComponentAsserts.ctl"

#uses "fwRPC/fwRPC_functionModel.ctl"
#uses "test/fwRPC/utils.ctl"

// Mocks / utils
//------------------------------------------------------------------------------

const string funcname = "func_name";
const string stubname = "stub_name";
const string libname = "lib_name";
const dyn_int paramtype = makeDynInt(FLOAT_VAR, INT_VAR);
bool exclusive = true;

const string dp = "_FwRPC_Function/func_name";


// Setup
//------------------------------------------------------------------------------

void test_fwRPC_functionModel_setupSuite() {
  _BEGIN_SUITE("function model");
}

void test_fwRPC_functionModel_teardownSuite() {
  _END_SUITE();
}

void test_fwRPC_functionModel_setup() {
  _BEGIN_TEST();
}

void test_fwRPC_functionModel_teardown() {
  if(dpExists(dp)) {
    dpDelete(dp);
  }
}


// tests
//------------------------------------------------------------------------------

void test_fwRPC_functionModel_createsValidObject() {
  dyn_mixed obj = fwRPC_functionModel_new(
    funcname, stubname, libname, paramtype, exclusive
  );

  assertEqual(fwRPC_FUNCTION_MODEL_MAXIDX, dynlen(obj));
  assertEqual(funcname, obj[fwRPC_FUNCTION_MODEL_NAME]);
  assertEqual(stubname, obj[fwRPC_FUNCTION_MODEL_STUB]);
  assertEqual(libname, obj[fwRPC_FUNCTION_MODEL_LIBFILE]);
  assertEqual(paramtype, obj[fwRPC_FUNCTION_MODEL_PARAMTYPE]);
  assertEqual(exclusive, obj[fwRPC_FUNCTION_MODEL_EXCLUSIVE]);
}


void test_fwRPC_functionModel_createsValidObjectFromQuery() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname);
  fwRPC_functionModel_store(obj);
  assertTrue(dpExists(obj[fwRPC_FUNCTION_MODEL_DP]), "dp should have been created");

  dyn_dyn_anytype results;
  dpQuery(fwRPC_FUNCTION_QUERY_UPDATE, results);
  assert(dynlen(results) >= 2);
  
  obj = makeDynMixed("");
  bool contains = false;
  for(int i=2; i <= dynlen(results) && !contains; i++) {
    obj = fwRPC_functionModel_newFromQuery(results[i]);
    if(obj[fwRPC_FUNCTION_MODEL_NAME] == funcname) contains = true;
  }

  assertTrue(contains);
}


void test_fwRPC_functionModel_generatesDP() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname);
  assertEqual(dp, obj[fwRPC_FUNCTION_MODEL_DP]);
}


void test_fwRPC_functionModel_checksValidity() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname);

  bool thrown = false;
  try {
    fwRPC_functionModel_checkValid(obj);
  } catch {
    thrown = true;
  }

  assertFalse(thrown);

  obj = makeDynMixed(1);
  thrown = false;

  try {
    fwRPC_functionModel_checkValid(obj);
  } catch {
    thrown = true;
  }

  assertTrue(thrown);
}


void test_fwRPC_functionModel_storesFunction() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname);

  bool thrown = false;
  try {
    fwRPC_functionModel_store(obj);
  } catch {
    thrown = true;
  }

  assertFalse(thrown);
  assertTrue(dpExists(dp));
}


void test_fwRPC_functionModel_doesntStoreBadFunction() {
  dyn_mixed obj = makeDynString("a");
  obj[fwRPC_FUNCTION_MODEL_DP] = dp;

  bool thrown = false;
  try {
    fwRPC_functionModel_store(obj);
  } catch {
    thrown = true;
  }

  assertTrue(thrown);

  assertFalse(dpExists(dp));
}


void test_fwRPC_functionModel_storeUpdatesFunctionDP() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname, "a");
  string stubname;

  fwRPC_functionModel_store(obj);

  dpGet(dp + fwRPC_FUNCTION_DPE_STUB, stubname);
  assertEqual("a", stubname);

  obj[fwRPC_FUNCTION_MODEL_STUB] = "b";
  fwRPC_functionModel_store(obj);

  dpGet(dp + fwRPC_FUNCTION_DPE_STUB, stubname);
  assertEqual("b", stubname);
}


void test_fwRPC_functionModel_syncsFunction() {
  dyn_mixed obj = fwRPC_functionModel_new(funcname, "a");
  
  fwRPC_functionModel_store(obj);
  
  obj[fwRPC_FUNCTION_MODEL_STUB] = "x";
  bool thrown = false;
  try {
    fwRPC_functionModel_sync(obj);
  } catch {
    thrown = true;
  }

  assertFalse(thrown);
  assertEqual("a", obj[fwRPC_FUNCTION_MODEL_STUB]);
}


void test_fwRPC_functionModel_doesntSyncBadFunction() {
  dyn_mixed obj = makeDynString("a");
  obj[fwRPC_FUNCTION_MODEL_DP] = dp;

  bool thrown = false;
  try {
    fwRPC_functionModel_sync(obj);
  } catch {
    thrown = true;
  }

  assertTrue(thrown);
  assertFalse(dpExists(dp));
}
