#uses "fwUnitTestComponentAsserts.ctl"

#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_msgClient.ctl"
#uses "fwRPC/fwRPC_common.ctl"

#uses "CtrlSerialization"
#uses "fwRPC/fwRPC_UUID.ctl"
#uses "fwRPC/fwRPC_RingBuffer.ctl"

#uses "CtrlZlib"

private global string _clientname = "name";
private global string _rbDp;
private global dyn_dyn_mixed _msgs;
private global string _ctx;

void test_fwRPC_msgClient_setupSuite() {
  _BEGIN_SUITE("msg client");

  _rbDp = fwRPC_RBMBX_DPT + "/" +
          fwRPC_DIST_TAG_LOCAL + "_" +
          fwRPC_DIST_TAG_CLIENT + "_" +
          _clientname;
}


void test_fwRPC_msgClient_teardownSuite() {
  _END_SUITE();
}


void test_fwRPC_msgClient_setup() {
  _BEGIN_TEST();

  dynClear(_msgs);
}

void test_fwRPC_msgClient_teardown() {
  fwRPC_msgClient_stop(_ctx);
}

void _writeToIn(string type, anytype payload) {
  string id = fwRPC_UUID_generateUUID4();

  string serialized;
  serializeToXml(payload, serialized);
  strreplace(serialized, "\n", "");

  string msg = id + " " + type +
                " " + fwRPC_DIST_BRKR +
                " " + fwRPC_DIST_BRKR +
                " " + serialized;

  int err = fwRPC_RingBuffer_write(_rbDp + ".in", msg);
  assertEqual(0, err, "error writing to rb");
}

void _mockDelegate(anytype ctx, string project, string name, anytype msg) {
  dynAppend(_msgs, makeDynMixed(project, name, msg));
}


//------------------------------------------------------------------------------


void test_fwRPC_msgClient_initialises_valid_param() {
  int err;
  
  mapping delegates;

  delegates["a"] = "a";
  
  err = fwRPC_msgClient_initialize(
    _ctx,
    true,
    fwRPC_DIST_TAG_CLIENT,
    _clientname,
    delegates
  );

  assertEqual(0, err);
  assertTrue(dpExists(_rbDp));
}


void test_fwRPC_msgClient_generates_name_if_none_specified() {
  int err;
  mapping delegates;
  delegates["a"] = "a";

  string name;
  err = fwRPC_msgClient_initialize(
    _ctx,
    true,
    fwRPC_DIST_TAG_CLIENT,
    name,
    delegates
  );

  assert(name != "", "name should be generated");
}

void test_fwRPC_msgClient_registers_delegates_and_receives() {
  int err;
  mapping delegates;

  delegates[fwRPC_DISTMSG_TYPE_ACKNOWLEDGE] = "_mockDelegate";

  err = fwRPC_msgClient_initialize(
    _ctx,
    true,
    fwRPC_DIST_TAG_CLIENT,
    _clientname,
    delegates
  );

  assertEqual(0, err, "initialise should succeed");

  _writeToIn(fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, "payload");

  delay(0, 500);

  assertEqual(1, dynlen(_msgs), "should have message");
  dyn_mixed msg = _msgs[1];
  assertEqual(fwRPC_DIST_BRKR, msg[1]);
  assertEqual(fwRPC_DIST_BRKR, msg[2]);
  assertEqual("payload", msg[3]);
}


void test_fwRPC_msgClient_initialise_reports_if_ctx_take_over() {
  int err;
  mapping del;
  del["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, del);
  assertEqual(0, err);
  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, del);
  assertEqual(1, err);
}


void test_fwRPC_msgClient_stop_fails_if_not_running() {
  int err = fwRPC_msgClient_stop(_ctx);
  assertEqual(-2, err);
}


void test_fwRPC_msgClient_stop_disconnects() {
  int err;
  mapping del;
  del[fwRPC_DISTMSG_TYPE_ACKNOWLEDGE] = "_mockDelegate";

  err = fwRPC_msgClient_initialize(
    _ctx,
    true,
    fwRPC_DIST_TAG_CLIENT,
    _clientname,
    del
  );
  assertEqual(0, err, "should initialize client");

  err = fwRPC_msgClient_stop(_ctx);
  assertEqual(0, err, "should stop client");

  _writeToIn(fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, "neverrecv");

  delay(0, 500);

  assertEqual(0, dynlen(_msgs), "should not receive message");
}


void test_fwRPC_msgClient_stop_sends_disconnect() {
  int err;
  mapping del;
  del["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, del);
  assertEqual(0, err);

  err = fwRPC_msgClient_stop(_ctx);
  assertEqual(0, err);

  string id, type, proj, name;
  anytype payload;
  _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);

  assertEqual(fwRPC_DISTMSG_TYPE_DISCONNECT, type);
  assertEqual(fwRPC_DIST_BRKR, proj);
  assertEqual(fwRPC_DIST_BRKR, name);
}


void test_fwRPC_msgClient_sends_message() {
  int err;
  mapping d; d["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, d);
  assertEqual(0, err);

  err = fwRPC_msgClient_send(
    _ctx,
    fwRPC_DISTMSG_TYPE_ACKNOWLEDGE,
    "a",
    "b",
    "asdf"
  );

  assertEqual(0, err);

  string id, type, proj, name; anytype payload;
  _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);

  assertEqual(fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, type);
  assertEqual("a", proj);
  assertEqual("b", name);
  assertEqual("asdf", payload);
}


void test_fwRPC_msgClient_send_fails_if_not_running() {
  int err = fwRPC_msgClient_send(
    _ctx,
    fwRPC_DISTMSG_TYPE_ACKNOWLEDGE,
    fwRPC_DIST_BRKR,
    fwRPC_DIST_BRKR,
    ""
  );

  assertEqual(-2, err);
}


void test_fwRPC_msgClient_sendRequest_sends_message() {
  int err;
  mapping d; d["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, d);
  assertEqual(0, err);

  err = fwRPC_msgClient_sendRequest(_ctx, fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, "asdf");
  assertEqual(0, err);

  string id, type, proj, name; anytype payload;
  _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);

  assertEqual(fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, type);
  assertEqual(fwRPC_DIST_BRKR, proj);
  assertEqual(fwRPC_DIST_BRKR, name);
  assertEqual("asdf", payload);
}


void test_fwRPC_msgClient_sendRequest_fails_if_not_running() {
  int err = fwRPC_msgClient_sendRequest(_ctx, fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, "");

  assertEqual(-2, err);
}


void test_fwRPC_msgClient_internal_buffer_if_rb_is_full() {
  const int bufferSize = 50;

  int err;
  mapping d; d["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, d);
  assertEqual(0, err);

  for (int i=0; i<100; ++i) {
    err = fwRPC_msgClient_sendRequest(_ctx, fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, i);
    assertEqual(0, err);
  }

  dyn_blob buffContents;
  dpGet(_rbDp + ".out" + fwRPC_RingBuffer_DPE_VALUES, buffContents);
  assertEqual(bufferSize, dynlen(buffContents), "buffer should be full");

  string id, type, proj, name, buffer; anytype payload;
  for (int i=1; i<=bufferSize; ++i) {
    string uncompressed;
    gunzip(buffContents[i], uncompressed);
    fwRPC_common_unpackMessage(uncompressed, id, type, proj, name, buffer);
    deserializeFromXml(buffer, payload);
    assertTrue(payload >= 0 && payload < bufferSize); //location is not guaranteed
  }

  _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);
  assertEqual(fwRPC_DISTMSG_TYPE_ACKNOWLEDGE, type);
  assertEqual(0, payload);

  delay(0, 500); //wait for drain to finish

  //buffer should now have #50
  dpGet(_rbDp + ".out" + fwRPC_RingBuffer_DPE_VALUES, buffContents);
  assertEqual(bufferSize, dynlen(buffContents));
  
  bool contains = false;
  for (int i=1; i<=bufferSize; ++i) {
    string uncompressed;
    gunzip(buffContents[i], uncompressed);
    fwRPC_common_unpackMessage(uncompressed, id, type, proj, name, buffer);
    deserializeFromXml(buffer, payload);
    if (payload == bufferSize) {
      contains = true;
      break;
    }
  }

  assertEqual(true, contains, "Buffer did not contain new value");

  //empty RB, nums should be #1 to #50
  for (int i=1; i<=bufferSize; ++i) {
    _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);
    assertEqual(true, payload > 0 && payload <= bufferSize, "value should be < 50");
  }

  //RB should contain #51 to #99
  for (int i=1; i<=bufferSize-1; ++i) {
    _getLastMessageFromRb(_rbDp, id, type, proj, name, payload);
    assertEqual(true, payload > bufferSize && payload < 100, "value should be >= 50");
  }

  //RB should now be empty
  err = fwRPC_RingBuffer_read(_rbDp + ".out", buffer);
  assertEqual(fwRPC_RingBuffer_EEMPTY, err, "RB should be empty");
}


void test_fwRPC_msgClient_disconnects_when_stopped_in_external_context() {
  int err;
  mapping d; d["a"] = "a";

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, d);
  assertEqual(0, err);

  evalScript(
    err,
    "int main() { return fwRPC_msgClient_stop(\""+_ctx+"\");}",
    makeDynString("")
  );

  assertEqual(0, err, "should stop client");

  err = fwRPC_msgClient_initialize(_ctx, true, fwRPC_DIST_TAG_CLIENT, _clientname, d);
  assertEqual(0, err, "should start client");
}
