#uses "fwUnitTestComponentAsserts.ctl"
#uses "test/fwRPC/utils.ctl"

#uses "fwRPC/fwRPC_worker.ctl"
#uses "fwRPC/fwRPC_RingBuffer.ctl"
#uses "fwRPC/fwRPC_UUID.ctl"
#uses "fwRPC/fwRPC_common.ctl"

void test_fwRPC_worker_setupSuite() {
  _BEGIN_SUITE("worker");

  fwRPC_registerFunction(
    _funcname, 
    testUtil_makeStubName(_funcname),
    _funclib
  );

  fwRPC_registerFunction(
    _funcErrParam,
    testUtil_makeStubName(_funcErrParam),
    _funclib
  );

  fwRPC_registerFunction(
    _funcErrValue,
    testUtil_makeStubName(_funcErrValue),
    _funclib
  );

  fwRPC_registerFunction(
    _funcErrNoValue,
    testUtil_makeStubName(_funcErrNoValue),
    _funclib
  );

  fwRPC_registerFunction(
    _funcTimeout,
    testUtil_makeStubName(_funcTimeout),
    _funclib,
    1
  );
}

void test_fwRPC_worker_teardownSuite() {
  _END_SUITE();
}

void test_fwRPC_worker_setup() {
  _BEGIN_TEST();
}

void test_fwRPC_worker_teardown() {
  if (fwRPC_worker_isRunning()) {
    int err = fwRPC_worker_stop();
    if (err < 0) DebugN("Warning: worker was still running at test end, could not halt");
  }
}


//------------------------------------------------------------------------------


private global const string _funclib = "test/fwRPC/test_fwRPC_worker.ctl";
private global const string _funcresult = "atgr";

private global const string _funcname = "f";

private global const string _funcErrParam = "f_err_param";
private global const string _funcErrValue = "f_err_value";
private global const string _funcErrNoValue = "f_err_noval";
private global const string _funcTimeout = "f_timeout";


anytype _f_stub(anytype hdl, anytype params) {
  delay(1);
  return fwRPC_makeResult(hdl, true, _funcresult);
}

anytype _f_err_param_stub(int x) {
  return fwRPC_makeResult(x, true, _funcresult);
}

anytype _f_err_value_stub(anytype hdl, anytype params) {
  return 1;
}

void _f_err_noval_stub(anytype hdl, anytype params) {
  //
}

anytype _f_timeout_stub(anytype hdl, anytype params) {
  delay(9);
  return fwRPC_makeResult(hdl, true, _funcresult);
}


bool _getLastWorkerMessage(
  string  rbDp,
  string  &id,
  string  &type,
  string  &proj,
  string  &name,
  anytype &payload
) {
  string val;
  string dp = rbDp + ".out";

  int err = fwRPC_RingBuffer_read(dp, val);
  if (err == fwRPC_RingBuffer_EEMPTY) return false;

  assertEqual(0, err, "Error reading DP: " + dp);

  string buffer;
  fwRPC_common_unpackMessage(val, id, type, proj, name, buffer);
  deserializeFromXml(buffer, payload);

  return true;
}


bool _waitForMessage(string rbDp, string type, anytype payload, int timeout=5000) {
  bool sent = false;
  string _id, _type, _proj, _name; anytype _payload;
  int elapsed = 0;

  while(elapsed < timeout) {
    _getLastWorkerMessage(rbDp, _id, _type, _proj, _name, _payload);
    if (
      _type == type && _dynEquals(_payload, payload)
    ) {
      sent = true;
      return true;
    }

    delay(0, 500);
    elapsed += 500;
  }

  return false;
}


bool _dynEquals(dyn_mixed a, dyn_mixed b) {
  if (dynlen(a) != dynlen(b)) return false;

  for (int i=1; i<=dynlen(a); ++i) {
    if (a[i] != b[i]) return false;
  }

  return true;
}


void _setupWorker(string &rbDp, int &workerId) {
  int err;

  assertEqual(0, fwRPC_worker_initialize(workerId));

  string workerName = fwRPC_DIST_TAG_LOCAL + "_" +
                      fwRPC_DIST_TAG_WORKER + "_" +
                      workerId;

  rbDp = fwRPC_common_makeDpName(fwRPC_RBMBX_DPT, workerName);
}


void _sendRequest(string rbDp, string &hdl) {
  _sendCustomRequest(rbDp, hdl, _funcname, "");
}


void _sendCustomRequest(string rbDp, string &hdl, string func, anytype params) {
  hdl = fwRPC_UUID_generateUUID4();

  dyn_mixed payload;
  payload[fwRPC_DISTMSG_REQUEST_HDL]    = hdl;
  payload[fwRPC_DISTMSG_REQUEST_FUNC]   = func;
  payload[fwRPC_DISTMSG_REQUEST_PARAMS] = params;

  string msg = fwRPC_dist_buildMessage(
    fwRPC_DISTMSG_TYPE_REQUEST,
    fwRPC_DIST_BRKR, fwRPC_DIST_BRKR,
    payload
  );

  int err = fwRPC_RingBuffer_write(rbDp + ".in", msg);
  assertEqual(0, err, "Error writing to dp: " + rbDp);
}

//------------------------------------------------------------------------------


void test_fwRPC_worker_start_stop() {
  assertFalse(fwRPC_worker_isRunning());

  int workerId;
  int err = fwRPC_worker_initialize(workerId);
  assertEqual(0, err);

  assertTrue(fwRPC_worker_isRunning());

  err = fwRPC_worker_initialize(workerId);
  assertTrue(err < 0);

  err = fwRPC_worker_stop();
  assertEqual(0, err);

  assertFalse(fwRPC_worker_isRunning());

  err = fwRPC_worker_stop();
  assertTrue(err < 0);

  assertFalse(fwRPC_worker_isRunning());
}


void test_fwRPC_worker_sends_disconnect_on_stop() {
  int workerId;
  string rbDp;
  _setupWorker(rbDp, workerId);

  int timeout = 5000;
  string id, type, proj, name; anytype payload;

  //stop the worker and expect a DISCONNECT msg from the msgClient
  assertEqual(0, fwRPC_worker_stop(), "Worker should stop");

  while(timeout > 0) {
    _getLastWorkerMessage(rbDp, id, type, proj, name, payload);

    if (type == fwRPC_DISTMSG_TYPE_DISCONNECT) break;

    delay(0, 500);
    timeout -= 500;
  }

  assertTrue(timeout > 0, "Timeout reached");
}


void test_fwRPC_worker_sends_heartbeat() {
  int err;
  int workerId;
  string rbDp;
  _setupWorker(rbDp, workerId);

  int hbcount = 0;
  int timeout = 5000;

  string id, type, proj, name; anytype payload;

  while(hbcount == 0 && timeout > 0) {
    _getLastWorkerMessage(rbDp, id, type, proj, name, payload);

    if (type == fwRPC_DISTMSG_TYPE_WORKER_HEARTBEAT) hbcount ++;

    delay(0, 500);
    timeout -= 500;
  }

  assertTrue(timeout > 0, "Timeout reached");

  assertTrue(hbcount > 0, "No heartbeat!");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_accepts_jobs_and_sends_status_updates() {
  int err;
  int workerId;
  string rbDp;

  _setupWorker(rbDp, workerId);

  string hdl;
  _sendRequest(rbDp, hdl);


  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_STATUS_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_STATUS_STATUS] = fwRPC_REQUEST_STATUS_PROCESSING;

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_STATUS, expectedPayload);
  assertTrue(sent, "worker should send status update on request");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_handles_invalid_stub_signature() {
  int workerId;
  string rbDp;
  _setupWorker(rbDp, workerId);

  anytype hdl;
  _sendCustomRequest(rbDp, hdl, _funcErrParam, "");

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_RESULT_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_RESULT_STATUS] = fwRPC_REQUEST_STATUS_ERROR;
  expectedPayload[fwRPC_DISTMSG_RESULT_OUTPUT] = makeError(
    getErrorCatalog(fwRPC_EX_WORKER_EXECUTION),
    getErrorPriority(fwRPC_EX_WORKER_EXECUTION),
    getErrorType(fwRPC_EX_WORKER_EXECUTION),
    getErrorCode(fwRPC_EX_WORKER_EXECUTION),
    "RPC function encountered an error: Invalid stub return value"
  );

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_SET_RESULT, expectedPayload);
  assertTrue(sent, "worker should send error result on bad stub signature");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_handles_invalid_stub_return_value() {
  int workerId;
  string rbDp;
  _setupWorker(rbDp, workerId);

  anytype hdl;
  _sendCustomRequest(rbDp, hdl, _funcErrValue, "");

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_RESULT_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_RESULT_STATUS] = fwRPC_REQUEST_STATUS_ERROR;
  expectedPayload[fwRPC_DISTMSG_RESULT_OUTPUT] = makeError(
    getErrorCatalog(fwRPC_EX_WORKER_EXECUTION),
    getErrorPriority(fwRPC_EX_WORKER_EXECUTION),
    getErrorType(fwRPC_EX_WORKER_EXECUTION),
    getErrorCode(fwRPC_EX_WORKER_EXECUTION),
    "RPC function encountered an error: Invalid stub return value"
  );

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_SET_RESULT, expectedPayload);
  assertTrue(sent, "worker should send error result on bad stub return value");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_handles_no_stub_return_value() {
  int workerId;
  string rbDp;
  _setupWorker(rbDp, workerId);

  anytype hdl;
  _sendCustomRequest(rbDp, hdl, _funcErrNoValue, "");

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_RESULT_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_RESULT_STATUS] = fwRPC_REQUEST_STATUS_ERROR;
  expectedPayload[fwRPC_DISTMSG_RESULT_OUTPUT] = makeError(
    getErrorCatalog(fwRPC_EX_WORKER_EXECUTION),
    getErrorPriority(fwRPC_EX_WORKER_EXECUTION),
    getErrorType(fwRPC_EX_WORKER_EXECUTION),
    getErrorCode(fwRPC_EX_WORKER_EXECUTION),
    "RPC function encountered an error: Invalid stub return value"
  );

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_SET_RESULT, expectedPayload);
  assertTrue(sent, "worker should send error result on no stub return value");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_cancels_jobs_and_sends_request_status() {
  int err;
  string rbDp;
  int workerId;

  _setupWorker(rbDp, workerId);

  string hdl;
  _sendRequest(rbDp, hdl);

  string msg = fwRPC_dist_buildMessage(
    fwRPC_DISTMSG_TYPE_CANCEL,
    fwRPC_DIST_BRKR, fwRPC_DIST_BRKR,
    hdl
  );

  delay(0,500);

  err = fwRPC_RingBuffer_write(rbDp + ".in", msg);
  assertEqual(0, err);

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_STATUS_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_STATUS_STATUS] = fwRPC_REQUEST_STATUS_CANCEL;

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_STATUS, expectedPayload);
  assertTrue(sent, "worker should send cancel status");

  fwRPC_worker_stop();
}

void test_fwRPC_worker_sends_request_result_on_completion() {
  int err;
  string rbDp;
  int workerId;

  _setupWorker(rbDp, workerId);

  string hdl;
  _sendRequest(rbDp, hdl);

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_RESULT_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_RESULT_STATUS] = fwRPC_REQUEST_STATUS_DONE;
  expectedPayload[fwRPC_DISTMSG_RESULT_OUTPUT] = _funcresult;

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_SET_RESULT, expectedPayload);
  assertTrue(sent, "worker should send request result on completion");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_times_out_function_execution() {
  int err;
  int workerId;
  string rbDp;
  string hdl;

  _setupWorker(rbDp, workerId);

  _sendCustomRequest(rbDp, hdl, _funcTimeout, "");

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_RESULT_HDL] = hdl;
  expectedPayload[fwRPC_DISTMSG_RESULT_STATUS] = fwRPC_REQUEST_STATUS_TIMEOUT;

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_STATUS, expectedPayload);
  assertTrue(sent, "worker should send request result on completion");

  fwRPC_worker_stop();
}


void test_fwRPC_worker_returns_worker_state_requests() {
  int err;

  int workerId;
  string rbDp;

  _setupWorker(rbDp, workerId);

  string msg = fwRPC_dist_buildMessage(
    fwRPC_DISTMSG_TYPE_WORKER_STATE,
    fwRPC_DIST_BRKR, fwRPC_DIST_BRKR,
    ""
  );

  err = fwRPC_RingBuffer_write(rbDp + ".in", msg);
  assertEqual(0, err);

  dyn_mixed expectedPayload = makeDynMixed();
  expectedPayload[fwRPC_DISTMSG_WORKER_STATE_ID] = workerId;
  expectedPayload[fwRPC_DISTMSG_WORKER_STATE_STATUS] = fwRPC_WORKER_STATUS_READY;
  expectedPayload[fwRPC_DISTMSG_WORKER_STATE_JOBS] = 0;
  expectedPayload[fwRPC_DISTMSG_WORKER_STATE_LOCKED] = false;

  bool sent = _waitForMessage(rbDp, fwRPC_DISTMSG_TYPE_WORKER_STATE, expectedPayload);
  assertTrue(sent, "Worker should respond to worker state requests");

  fwRPC_worker_stop();
}

